@@ -37,8 +37,6 @@
 #include <spa/pod/filter.h>
 #include <spa/debug/pod.h>
 
-#define NAME "alsa-pcm-sink"
-
 #include "alsa-pcm.h"
 
 #define CHECK_PORT(this,d,p)    ((d) == SPA_DIRECTION_INPUT && (p) == 0)
@@ -143,64 +141,63 @@ static int impl_node_enum_params(void *object, int seq,
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_device),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA device"),
+				SPA_PROP_INFO_name, SPA_POD_String(SPA_KEY_API_ALSA_PATH),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA device"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->device, sizeof(p->device)));
 			break;
 		case 1:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_deviceName),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA device name"),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA device name"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->device_name, sizeof(p->device_name)));
 			break;
 		case 2:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_cardName),
-				SPA_PROP_INFO_name, SPA_POD_String("The ALSA card name"),
+				SPA_PROP_INFO_description, SPA_POD_String("The ALSA card name"),
 				SPA_PROP_INFO_type, SPA_POD_Stringn(p->card_name, sizeof(p->card_name)));
 			break;
 		case 3:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_minLatency),
-				SPA_PROP_INFO_name, SPA_POD_String("The minimum latency"),
+				SPA_PROP_INFO_description, SPA_POD_String("The minimum latency"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(p->min_latency, 1, INT32_MAX));
 			break;
 		case 4:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_maxLatency),
-				SPA_PROP_INFO_name, SPA_POD_String("The maximum latency"),
+				SPA_PROP_INFO_description, SPA_POD_String("The maximum latency"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Int(p->max_latency, 1, INT32_MAX));
 			break;
 		case 5:
-			param = spa_pod_builder_add_object(&b,
-				SPA_TYPE_OBJECT_PropInfo, id,
-				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_START_CUSTOM),
-				SPA_PROP_INFO_name, SPA_POD_String("Use the driver channelmap"),
-				SPA_PROP_INFO_type, SPA_POD_Bool(p->use_chmap));
-			break;
-		case 6:
 			param = spa_pod_builder_add_object(&b,
 				SPA_TYPE_OBJECT_PropInfo, id,
 				SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_latencyOffsetNsec),
-				SPA_PROP_INFO_name, SPA_POD_String("Latency offset (ns)"),
+				SPA_PROP_INFO_description, SPA_POD_String("Latency offset (ns)"),
 				SPA_PROP_INFO_type, SPA_POD_CHOICE_RANGE_Long(0LL, 0LL, INT64_MAX));
 			break;
-		case 7:
+		case 6:
 			if (this->is_iec958 || this->is_hdmi) {
 				param = spa_pod_builder_add_object(&b,
 					SPA_TYPE_OBJECT_PropInfo, id,
 					SPA_PROP_INFO_id,   SPA_POD_Id(SPA_PROP_iec958Codecs),
-					SPA_PROP_INFO_name, SPA_POD_String("Enabled IEC958 (S/PDIF) codecs"),
+					SPA_PROP_INFO_name, SPA_POD_String("iec958.codecs"),
+					SPA_PROP_INFO_description, SPA_POD_String("Enabled IEC958 (S/PDIF) codecs"),
 					SPA_PROP_INFO_type, SPA_POD_Id(SPA_AUDIO_IEC958_CODEC_UNKNOWN),
+	                                SPA_PROP_INFO_params, SPA_POD_Bool(true),
 	                                SPA_PROP_INFO_container, SPA_POD_Id(SPA_TYPE_Array));
+
 				break;
 			}
 			SPA_FALLTHROUGH
 		default:
-			return 0;
+			param = spa_alsa_enum_propinfo(this, result.index - 7, &b);
+			if (param == NULL)
+				return 0;
 		}
 		break;
 	}
@@ -220,7 +217,6 @@ static int impl_node_enum_params(void *object, int seq,
 				SPA_PROP_cardName,     SPA_POD_Stringn(p->card_name, sizeof(p->card_name)),
 				SPA_PROP_minLatency,   SPA_POD_Int(p->min_latency),
 				SPA_PROP_maxLatency,   SPA_POD_Int(p->max_latency),
-				SPA_PROP_START_CUSTOM, SPA_POD_Bool(p->use_chmap),
 				SPA_PROP_latencyOffsetNsec,   SPA_POD_Long(this->process_latency.ns),
 				0);
 
@@ -230,6 +226,7 @@ static int impl_node_enum_params(void *object, int seq,
 				spa_pod_builder_array(&b, sizeof(uint32_t), SPA_TYPE_Id,
 						n_codecs, codecs);
 			}
+			spa_alsa_add_prop_params(this, &b);
 			param = spa_pod_builder_pop(&b, &f);
 			break;
 		default:
@@ -336,7 +333,7 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 	{
 		struct props *p = &this->props;
 		struct spa_process_latency_info info;
-		struct spa_pod *iec958_codecs = NULL;
+		struct spa_pod *iec958_codecs = NULL, *params = NULL;
 
 		if (param == NULL) {
 			reset_props(p);
@@ -351,8 +348,8 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 			SPA_PROP_minLatency,   SPA_POD_OPT_Int(&p->min_latency),
 			SPA_PROP_maxLatency,   SPA_POD_OPT_Int(&p->max_latency),
 			SPA_PROP_latencyOffsetNsec,   SPA_POD_OPT_Long(&info.ns),
-			SPA_PROP_START_CUSTOM, SPA_POD_OPT_Bool(&p->use_chmap),
-			SPA_PROP_iec958Codecs, SPA_POD_OPT_Pod(&iec958_codecs));
+			SPA_PROP_iec958Codecs, SPA_POD_OPT_Pod(&iec958_codecs),
+			SPA_PROP_params,       SPA_POD_OPT_Pod(&params));
 
 		if ((this->is_iec958 || this->is_hdmi) && iec958_codecs != NULL) {
 			uint32_t i, codecs[16], n_codecs;
@@ -369,6 +366,7 @@ static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
 			this->port_params[PORT_EnumFormat].user++;
 		}
 		handle_process_latency(this, &info);
+		spa_alsa_parse_prop_params(this, params);
 
 		emit_node_info(this, false);
 		emit_port_info(this, false);
@@ -535,6 +533,10 @@ impl_node_port_enum_params(void *object, int seq,
 			param = spa_format_audio_iec958_build(&b, id,
 					&this->current_format.info.iec958);
 			break;
+		case SPA_MEDIA_SUBTYPE_dsd:
+			param = spa_format_audio_dsd_build(&b, id,
+					&this->current_format.info.dsd);
+			break;
 		default:
 			return -EIO;
 		}
@@ -642,10 +644,8 @@ static int port_set_format(void *object,
 			return 0;
 
 		spa_log_debug(this->log, "clear format");
-		spa_alsa_pause(this);
-		clear_buffers(this);
 		spa_alsa_close(this);
-		this->have_format = false;
+		clear_buffers(this);
 	} else {
 		struct spa_audio_info info = { 0 };
 
@@ -664,6 +664,10 @@ static int port_set_format(void *object,
 			if (spa_format_audio_iec958_parse(format, &info.info.iec958) < 0)
 				return -EINVAL;
 			break;
+		case SPA_MEDIA_SUBTYPE_dsd:
+			if (spa_format_audio_dsd_parse(format, &info.info.dsd) < 0)
+				return -EINVAL;
+			break;
 		default:
 			return -EINVAL;
 		}
@@ -672,7 +676,6 @@ static int port_set_format(void *object,
 			return err;
 
 		this->current_format = info;
-		this->have_format = true;
 	}
 
 	this->info.change_mask |= SPA_NODE_CHANGE_MASK_PROPS;
@@ -745,7 +748,7 @@ impl_node_port_use_buffers(void *object,
 
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: use %d buffers", this, n_buffers);
+	spa_log_debug(this->log, "%p: use %d buffers", this, n_buffers);
 
 	if (!this->have_format)
 		return -EIO;
@@ -767,10 +770,10 @@ impl_node_port_use_buffers(void *object,
 		b->h = spa_buffer_find_meta_data(b->buf, SPA_META_Header, sizeof(*b->h));
 
 		if (d[0].data == NULL) {
-			spa_log_error(this->log, NAME " %p: need mapped memory", this);
+			spa_log_error(this->log, "%p: need mapped memory", this);
 			return -EINVAL;
 		}
-		spa_log_debug(this->log, NAME " %p: %d %p data:%p", this, i, b->buf, d[0].data);
+		spa_log_debug(this->log, "%p: %d %p data:%p", this, i, b->buf, d[0].data);
 	}
 	this->n_buffers = n_buffers;
 
@@ -790,7 +793,7 @@ impl_node_port_set_io(void *object,
 
 	spa_return_val_if_fail(CHECK_PORT(this, direction, port_id), -EINVAL);
 
-	spa_log_debug(this->log, NAME " %p: io %d %p %zd", this, id, data, size);
+	spa_log_debug(this->log, "%p: io %d %p %zd", this, id, data, size);
 
 	switch (id) {
 	case SPA_IO_Buffers:
@@ -820,7 +823,7 @@ static int impl_node_process(void *object)
 	input = this->io;
 	spa_return_val_if_fail(input != NULL, -EIO);
 
-	spa_log_trace_fp(this->log, NAME " %p: process %d %d/%d", this, input->status,
+	spa_log_trace_fp(this->log, "%p: process %d %d/%d", this, input->status,
 			input->buffer_id, this->n_buffers);
 
 	if (this->position && this->position->clock.flags & SPA_IO_CLOCK_FLAG_FREEWHEEL) {
@@ -832,12 +835,12 @@ static int impl_node_process(void *object)
 		struct buffer *b = &this->buffers[input->buffer_id];
 
 		if (!SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_OUT)) {
-			spa_log_warn(this->log, NAME " %p: buffer %u in use",
+			spa_log_warn(this->log, "%p: buffer %u in use",
 					this, input->buffer_id);
 			input->status = -EINVAL;
 			return -EINVAL;
 		}
-		spa_log_trace_fp(this->log, NAME " %p: queue buffer %u", this, input->buffer_id);
+		spa_log_trace_fp(this->log, "%p: queue buffer %u", this, input->buffer_id);
 		spa_list_append(&this->ready, &b->link);
 		SPA_FLAG_CLEAR(b->flags, BUFFER_FLAG_OUT);
 		input->buffer_id = SPA_ID_INVALID;
@@ -907,7 +910,6 @@ impl_init(const struct spa_handle_factory *factory,
 	  struct spa_handle *handle, const struct spa_dict *info, const struct spa_support *support, uint32_t n_support)
 {
 	struct state *this;
-	uint32_t i;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
 	spa_return_val_if_fail(handle != NULL, -EINVAL);
@@ -918,6 +920,8 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct state *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	alsa_log_topic_init(this->log);
+
 	this->data_system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataSystem);
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 
@@ -977,45 +981,7 @@ impl_init(const struct spa_handle_factory *factory,
 
 	spa_list_init(&this->ready);
 
-	for (i = 0; info && i < info->n_items; i++) {
-		const char *k = info->items[i].key;
-		const char *s = info->items[i].value;
-		if (spa_streq(k, SPA_KEY_API_ALSA_PATH)) {
-			snprintf(this->props.device, 63, "%s", s);
-		} else if (spa_streq(k, SPA_KEY_API_ALSA_PCM_CARD)) {
-			this->card_index = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_API_ALSA_OPEN_UCM)) {
-			this->open_ucm = spa_atob(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_CHANNELS)) {
-			this->default_channels = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_RATE)) {
-			this->default_rate = atoi(s);
-		} else if (spa_streq(k, SPA_KEY_AUDIO_FORMAT)) {
-			this->default_format = spa_alsa_format_from_name(s, strlen(s));
-		} else if (spa_streq(k, SPA_KEY_AUDIO_POSITION)) {
-			spa_alsa_parse_position(&this->default_pos, s, strlen(s));
-		} else if (spa_streq(k, "latency.internal.rate")) {
-			this->process_latency.rate = atoi(s);
-		} else if (spa_streq(k, "latency.internal.ns")) {
-			this->process_latency.ns = atoi(s);
-		} else if (spa_streq(k, "iec958.codecs")) {
-			spa_alsa_parse_iec958_codecs(&this->iec958_codecs, s, strlen(s));
-		} else if (spa_streq(k, "api.alsa.period-size")) {
-			this->default_period_size = atoi(s);
-		} else if (spa_streq(k, "api.alsa.headroom")) {
-			this->default_headroom = atoi(s);
-		} else if (spa_streq(k, "api.alsa.start-delay")) {
-			this->default_start_delay = atoi(s);
-		} else if (spa_streq(k, "api.alsa.disable-mmap")) {
-			this->disable_mmap = spa_atob(s);
-		} else if (spa_streq(k, "api.alsa.disable-batch")) {
-			this->disable_batch = spa_atob(s);
-		} else if (spa_streq(k, "api.alsa.use-chmap")) {
-			this->props.use_chmap = spa_atob(s);
-		}
-	}
-
-	return spa_alsa_init(this);
+	return spa_alsa_init(this, info);
 }
 
 static const struct spa_interface_info impl_interfaces[] = {
