@@ -34,6 +34,8 @@
 #include <spa/support/loop.h>
 #include <spa/utils/list.h>
 #include <spa/utils/keys.h>
+#include <spa/utils/json.h>
+#include <spa/utils/string.h>
 #include <spa/node/node.h>
 #include <spa/node/utils.h>
 #include <spa/node/io.h>
@@ -47,11 +49,14 @@
 
 #define NAME "null-audio-sink"
 
+#define DEFAULT_CLOCK_NAME	"clock.system.monotonic"
+
 struct props {
 	uint32_t channels;
 	uint32_t rate;
 	uint32_t n_pos;
 	uint32_t pos[SPA_AUDIO_MAX_CHANNELS];
+	char clock_name[64];
 };
 
 static void reset_props(struct props *props)
@@ -59,6 +64,7 @@ static void reset_props(struct props *props)
 	props->channels = 0;
 	props->rate = 0;
 	props->n_pos = 0;
+	strncpy(props->clock_name, DEFAULT_CLOCK_NAME, sizeof(props->clock_name));
 }
 
 #define DEFAULT_CHANNELS	2
@@ -115,6 +121,7 @@ struct impl {
 	struct port port;
 
 	unsigned int started:1;
+	unsigned int following:1;
 	struct spa_source timer_source;
 	struct itimerspec timerspec;
 	uint64_t next_time;
@@ -179,6 +186,65 @@ static int impl_node_enum_params(void *object, int seq,
 	return 0;
 }
 
+static void set_timeout(struct impl *this, uint64_t next_time)
+{
+	spa_log_trace(this->log, "set timeout %"PRIu64, next_time);
+	this->timerspec.it_value.tv_sec = next_time / SPA_NSEC_PER_SEC;
+	this->timerspec.it_value.tv_nsec = next_time % SPA_NSEC_PER_SEC;
+	spa_system_timerfd_settime(this->data_system,
+			this->timer_source.fd, SPA_FD_TIMER_ABSTIME, &this->timerspec, NULL);
+}
+
+static int set_timers(struct impl *this)
+{
+	struct timespec now;
+	int res;
+
+	if ((res = spa_system_clock_gettime(this->data_system, CLOCK_MONOTONIC, &now)) < 0)
+	    return res;
+	this->next_time = SPA_TIMESPEC_TO_NSEC(&now);
+
+	if (this->following) {
+		set_timeout(this, 0);
+	} else {
+		set_timeout(this, this->next_time);
+	}
+	return 0;
+}
+
+static inline bool is_following(struct impl *this)
+{
+	return this->position && this->clock && this->position->clock.id != this->clock->id;
+}
+
+static int do_reassign_follower(struct spa_loop *loop,
+			    bool async,
+			    uint32_t seq,
+			    const void *data,
+			    size_t size,
+			    void *user_data)
+{
+	struct impl *this = user_data;
+	set_timers(this);
+	return 0;
+}
+
+static int reassign_follower(struct impl *this)
+{
+	bool following;
+
+	if (!this->started)
+		return 0;
+
+	following = is_following(this);
+	if (following != this->following) {
+		spa_log_debug(this->log, NAME" %p: reassign follower %d->%d", this, this->following, following);
+		this->following = following;
+		spa_loop_invoke(this->data_loop, do_reassign_follower, 0, NULL, 0, true, this);
+	}
+	return 0;
+}
+
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	struct impl *this = object;
@@ -190,6 +256,11 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 		if (size > 0 && size < sizeof(struct spa_io_clock))
 			return -EINVAL;
 		this->clock = data;
+		if (this->clock != NULL) {
+			spa_scnprintf(this->clock->name,
+					sizeof(this->clock->name),
+					"%s", this->props.clock_name);
+		}
 		break;
 	case SPA_IO_Position:
 		this->position = data;
@@ -197,19 +268,11 @@ static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 	default:
 		return -ENOENT;
 	}
+	reassign_follower(this);
+
 	return 0;
 }
 
-static void set_timer(struct impl *this, uint64_t next_time)
-{
-	spa_log_trace(this->log, "set timer %"PRIu64, next_time);
-
-	this->timerspec.it_value.tv_sec = next_time / SPA_NSEC_PER_SEC;
-	this->timerspec.it_value.tv_nsec = next_time % SPA_NSEC_PER_SEC;
-	spa_system_timerfd_settime(this->data_system,
-			this->timer_source.fd, SPA_FD_TIMER_ABSTIME, &this->timerspec, NULL);
-}
-
 static void on_timeout(struct spa_source *source)
 {
 	struct impl *this = source->data;
@@ -219,8 +282,11 @@ static void on_timeout(struct spa_source *source)
 	spa_log_trace(this->log, "timeout");
 
 	if (spa_system_timerfd_read(this->data_system,
-				this->timer_source.fd, &expirations) < 0)
+				this->timer_source.fd, &expirations) < 0) {
+		if (errno == EAGAIN)
+			return;
 		perror("read timerfd");
+	}
 
 	nsec = this->next_time;
 
@@ -245,7 +311,27 @@ static void on_timeout(struct spa_source *source)
 
 	spa_node_call_ready(&this->callbacks, SPA_STATUS_NEED_DATA);
 
-	set_timer(this, this->next_time);
+	set_timeout(this, this->next_time);
+}
+
+static int do_start(struct impl *this)
+{
+	if (this->started)
+		return 0;
+
+	this->following = is_following(this);
+	set_timers(this);
+	this->started = true;
+	return 0;
+}
+
+static int do_stop(struct impl *this)
+{
+	if (!this->started)
+		return 0;
+	this->started = false;
+	set_timeout(this, 0);
+	return 0;
 }
 
 static int impl_node_send_command(void *object, const struct spa_command *command)
@@ -261,28 +347,17 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 	switch (SPA_NODE_COMMAND_ID(command)) {
 	case SPA_NODE_COMMAND_Start:
 	{
-		struct timespec now;
-
 		if (!port->have_format)
 			return -EIO;
 		if (port->n_buffers == 0)
 			return -EIO;
 
-		if (this->started)
-			return 0;
-
-		clock_gettime(CLOCK_MONOTONIC, &now);
-		this->next_time = SPA_TIMESPEC_TO_NSEC(&now);
-		set_timer(this, this->next_time);
-		this->started = true;
+		do_start(this);
 		break;
 	}
 	case SPA_NODE_COMMAND_Suspend:
 	case SPA_NODE_COMMAND_Pause:
-		if (!this->started)
-			return 0;
-		this->started = false;
-		set_timer(this, 0);
+		do_stop(this);
 		break;
 
 	default:
@@ -297,23 +372,25 @@ static const struct spa_dict_item node_info_items[] = {
 
 static void emit_node_info(struct impl *this, bool full)
 {
+	uint64_t old = full ? this->info.change_mask : 0;
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
 		this->info.props = &SPA_DICT_INIT_ARRAY(node_info_items);
 		spa_node_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
 static void emit_port_info(struct impl *this, struct port *port, bool full)
 {
+	uint64_t old = full ? port->info.change_mask : 0;
 	if (full)
 		port->info.change_mask = port->info_all;
 	if (port->info.change_mask) {
 		spa_node_emit_port_info(&this->hooks,
 				SPA_DIRECTION_INPUT, 0, &port->info);
-		port->info.change_mask = 0;
+		port->info.change_mask = old;
 	}
 }
 
@@ -389,7 +466,7 @@ port_enum_formats(struct impl *this,
 				0);
 		} else {
 			spa_pod_builder_add(builder,
-				SPA_FORMAT_AUDIO_rate, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX),
+				SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX),
 				0);
 		}
 		if (this->props.n_pos != 0) {
@@ -694,7 +771,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Node))
 		*interface = &this->node;
 	else
 		return -ENOENT;
@@ -723,16 +800,32 @@ impl_get_size(const struct spa_handle_factory *factory,
 	return sizeof(struct impl);
 }
 
-static uint32_t channel_from_name(const char *name, size_t len)
+static uint32_t channel_from_name(const char *name)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (strncmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name), len) == 0)
+		if (spa_streq(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)))
 			return spa_type_audio_channel[i].type;
 	}
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
 }
 
+static inline void parse_position(struct impl *this, const char *val, size_t len)
+{
+	struct spa_json it[2];
+	char v[256];
+
+	spa_json_init(&it[0], val, len);
+        if (spa_json_enter_array(&it[0], &it[1]) <= 0)
+                spa_json_init(&it[1], val, len);
+
+	this->props.n_pos = 0;
+	while (spa_json_get_string(&it[1], v, sizeof(v)) > 0 &&
+	    this->props.n_pos < SPA_AUDIO_MAX_CHANNELS) {
+		this->props.pos[this->props.n_pos++] = channel_from_name(v);
+	}
+}
+
 static int
 impl_init(const struct spa_handle_factory *factory,
 	  struct spa_handle *handle,
@@ -798,7 +891,8 @@ impl_init(const struct spa_handle_factory *factory,
 
 	this->timer_source.func = on_timeout;
 	this->timer_source.data = this;
-	this->timer_source.fd = spa_system_timerfd_create(this->data_system, CLOCK_MONOTONIC, SPA_FD_CLOEXEC);
+	this->timer_source.fd = spa_system_timerfd_create(this->data_system, CLOCK_MONOTONIC,
+							  SPA_FD_CLOEXEC | SPA_FD_NONBLOCK);
 	this->timer_source.mask = SPA_IO_IN;
 	this->timer_source.rmask = 0;
 	this->timerspec.it_value.tv_sec = 0;
@@ -809,19 +903,18 @@ impl_init(const struct spa_handle_factory *factory,
 	spa_loop_add_source(this->data_loop, &this->timer_source);
 
 	for (i = 0; info && i < info->n_items; i++) {
-		if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_CHANNELS)) {
-			this->props.channels = atoi(info->items[i].value);
-		} else if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_RATE)) {
-			this->props.rate = atoi(info->items[i].value);
-		} else if (!strcmp(info->items[i].key, SPA_KEY_AUDIO_POSITION)) {
-			size_t len;
-			const char *p = info->items[i].value;
-			while (*p && this->props.n_pos < SPA_AUDIO_MAX_CHANNELS) {
-				if ((len = strcspn(p, ",")) == 0)
-					break;
-				this->props.pos[this->props.n_pos++] = channel_from_name(p, len);
-				p += len + strspn(p+len, ",");
-			}
+		const char *k = info->items[i].key;
+		const char *s = info->items[i].value;
+		if (spa_streq(k, SPA_KEY_AUDIO_CHANNELS)) {
+			this->props.channels = atoi(s);
+		} else if (spa_streq(k, SPA_KEY_AUDIO_RATE)) {
+			this->props.rate = atoi(s);
+		} else if (spa_streq(k, SPA_KEY_AUDIO_POSITION)) {
+			parse_position(this, s, strlen(s));
+		} else if (spa_streq(k, "clock.name")) {
+			spa_scnprintf(this->props.clock_name,
+					sizeof(this->props.clock_name),
+					"%s", s);
 		}
 	}
 	if (this->props.n_pos > 0)
