@@ -49,7 +49,7 @@ struct builtin {
 };
 
 static void *builtin_instantiate(const struct fc_descriptor * Descriptor,
-		unsigned long SampleRate, int index, const char *config)
+		unsigned long *SampleRate, int index, const char *config)
 {
 	struct builtin *impl;
 
@@ -57,7 +57,7 @@ static void *builtin_instantiate(const struct fc_descriptor * Descriptor,
 	if (impl == NULL)
 		return NULL;
 
-	impl->rate = SampleRate;
+	impl->rate = *SampleRate;
 
 	return impl;
 }
@@ -281,7 +281,7 @@ static void bq_run(struct builtin *impl, unsigned long samples, int type)
 		impl->freq = freq;
 		impl->Q = Q;
 		impl->gain = gain;
-		biquad_set(bq, type, freq / impl->rate, Q, gain);
+		biquad_set(bq, type, freq * 2 / impl->rate, Q, gain);
 	}
 	x1 = bq->x1;
 	x2 = bq->x2;
@@ -469,7 +469,7 @@ struct convolver_impl {
 };
 
 static float *read_samples(const char *filename, float gain, int delay, int offset,
-		int length, int channel, int *n_samples)
+		int length, int channel, long unsigned *rate, int *n_samples)
 {
 	float *samples;
 #ifdef HAVE_SNDFILE
@@ -510,6 +510,7 @@ static float *read_samples(const char *filename, float gain, int delay, int offs
 		samples[i] = samples[info.channels * i + channel] * gain;
 
 	*n_samples = n;
+	*rate = info.samplerate;
 	return samples;
 #else
 	pw_log_error("compiled without sndfile support, can't load samples: "
@@ -570,7 +571,7 @@ static float *create_dirac(const char *filename, float gain, int delay, int offs
 }
 
 static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
-		unsigned long SampleRate, int index, const char *config)
+		unsigned long *SampleRate, int index, const char *config)
 {
 	struct convolver_impl *impl;
 	float *samples;
@@ -642,7 +643,7 @@ static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
 				length, &n_samples);
 	} else {
 		samples = read_samples(filename, gain, delay, offset,
-				length, channel, &n_samples);
+				length, channel, SampleRate, &n_samples);
 	}
 	if (samples == NULL)
 		return NULL;
@@ -656,14 +657,21 @@ static void * convolver_instantiate(const struct fc_descriptor * Descriptor,
 
 	impl = calloc(1, sizeof(*impl));
 	if (impl == NULL)
-		return NULL;
+		goto error;
 
-	impl->rate = SampleRate;
+	impl->rate = *SampleRate;
 
 	impl->conv = convolver_new(blocksize, tailsize, samples, n_samples);
+	if (impl->conv == NULL)
+		goto error;
+
 	free(samples);
 
 	return impl;
+error:
+	free(samples);
+	free(impl);
+	return NULL;
 }
 
 static void convolver_connect_port(void * Instance, unsigned long Port,
@@ -676,6 +684,8 @@ static void convolver_connect_port(void * Instance, unsigned long Port,
 static void convolver_cleanup(void * Instance)
 {
 	struct convolver_impl *impl = Instance;
+	if (impl->conv)
+		convolver_free(impl->conv);
 	free(impl);
 }
 
@@ -690,6 +700,12 @@ static struct fc_port convolve_ports[] = {
 	},
 };
 
+static void convolver_deactivate(void * Instance)
+{
+	struct convolver_impl *impl = Instance;
+	convolver_reset(impl->conv);
+}
+
 static void convolve_run(void * Instance, unsigned long SampleCount)
 {
 	struct convolver_impl *impl = Instance;
@@ -704,6 +720,7 @@ static const struct fc_descriptor convolve_desc = {
 
 	.instantiate = convolver_instantiate,
 	.connect_port = convolver_connect_port,
+	.deactivate = convolver_deactivate,
 	.run = convolve_run,
 	.cleanup = convolver_cleanup,
 };
