@@ -36,20 +36,23 @@
 
 #include <pipewire/pipewire.h>
 
-#define spa_debug pw_log_debug
+PW_LOG_TOPIC_EXTERN(mod_topic);
+#define PW_LOG_TOPIC_DEFAULT mod_topic
+PW_LOG_TOPIC_EXTERN(mod_topic_connection);
+
+#undef spa_debug
+#define spa_debug(...) pw_logt_debug(mod_topic_connection, __VA_ARGS__)
 #include <spa/debug/pod.h>
 
 #include "connection.h"
 
 #define MAX_BUFFER_SIZE (1024 * 32)
-#define MAX_FDS 1024
+#define MAX_FDS 1024u
 #define MAX_FDS_MSG 28
 
 #define HDR_SIZE_V0	8
 #define HDR_SIZE	16
 
-static bool debug_messages = 0;
-
 struct buffer {
 	uint8_t *buffer_data;
 	size_t buffer_size;
@@ -164,6 +167,14 @@ static void *connection_ensure_size(struct pw_protocol_native_connection *conn,
 	return (uint8_t *) buf->buffer_data + buf->buffer_size;
 }
 
+static void handle_connection_error(struct pw_protocol_native_connection *conn, int res)
+{
+	if (res == -EPIPE || res == -ECONNRESET)
+		pw_log_info("connection %p: could not recvmsg on fd:%d: %s", conn, conn->fd, strerror(res));
+	else
+		pw_log_error("connection %p: could not recvmsg on fd:%d: %s", conn, conn->fd, strerror(res));
+}
+
 static int refill_buffer(struct pw_protocol_native_connection *conn, struct buffer *buf)
 {
 	ssize_t len;
@@ -186,6 +197,8 @@ static int refill_buffer(struct pw_protocol_native_connection *conn, struct buff
 
 	while (true) {
 		len = recvmsg(conn->fd, &msg, msg.msg_flags);
+		if (msg.msg_flags & MSG_CTRUNC)
+			return -EPROTO;
 		if (len == 0 && avail != 0)
 			return -EPIPE;
 		else if (len < 0) {
@@ -207,6 +220,8 @@ static int refill_buffer(struct pw_protocol_native_connection *conn, struct buff
 
 		n_fds =
 		    (cmsg->cmsg_len - ((char *) CMSG_DATA(cmsg) - (char *) cmsg)) / sizeof(int);
+		if (n_fds + buf->n_fds > MAX_FDS)
+			return -EPROTO;
 		memcpy(&buf->fds[buf->n_fds], CMSG_DATA(cmsg), n_fds * sizeof(int));
 		buf->n_fds += n_fds;
 	}
@@ -217,7 +232,7 @@ static int refill_buffer(struct pw_protocol_native_connection *conn, struct buff
 
 	/* ERRORS */
 recv_error:
-	pw_log_error("connection %p: could not recvmsg on fd:%d: %m", conn, conn->fd);
+	handle_connection_error(conn, errno);
 	return -errno;
 }
 
@@ -363,7 +378,6 @@ struct pw_protocol_native_connection *pw_protocol_native_connection_new(struct p
 	if (impl == NULL)
 		return NULL;
 
-	debug_messages = pw_debug_is_category_enabled("connection");
 	impl->context = context;
 
 	this = &impl->this;
@@ -477,6 +491,9 @@ static int prepare_packet(struct pw_protocol_native_connection *conn, struct buf
 	size -= impl->hdr_size;
 	buf->msg.fds = &buf->fds[buf->fds_offset];
 
+	if (buf->msg.n_fds + buf->fds_offset > MAX_FDS)
+		return -EPROTO;
+
 	if (size < len)
 		return len;
 
@@ -556,7 +573,7 @@ static inline void *begin_write(struct pw_protocol_native_connection *conn, uint
 	if ((p = connection_ensure_size(conn, buf, impl->hdr_size + size)) == NULL)
 		return NULL;
 
-	return SPA_MEMBER(p, impl->hdr_size, void);
+	return SPA_PTROFF(p, impl->hdr_size, void);
 }
 
 static int builder_overflow(void *data, uint32_t size)
@@ -626,10 +643,10 @@ pw_protocol_native_connection_end(struct pw_protocol_native_connection *conn,
 	else
 		buf->n_fds = buf->msg.n_fds;
 
-	if (debug_messages) {
+	if (mod_topic_connection->level >= SPA_LOG_LEVEL_DEBUG) {
 		pw_log_debug(">>>>>>>>> out: id:%d op:%d size:%d seq:%d",
 				buf->msg.id, buf->msg.opcode, size, buf->msg.seq);
-	        spa_debug_pod(0, NULL, SPA_MEMBER(p, impl->hdr_size, struct spa_pod));
+	        spa_debug_pod(0, NULL, SPA_PTROFF(p, impl->hdr_size, struct spa_pod));
 	}
 
 	buf->seq = (buf->seq + 1) & SPA_ASYNC_SEQ_MASK;
@@ -717,7 +734,7 @@ int pw_protocol_native_connection_flush(struct pw_protocol_native_connection *co
 			     outfds);
 
 		size -= sent;
-		data = SPA_MEMBER(data, sent, void);
+		data = SPA_PTROFF(data, sent, void);
 		n_fds -= outfds;
 		fds += outfds;
 		to_close += outfds;
