@@ -25,17 +25,12 @@
 #include "config.h"
 
 #include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
 #ifndef ENODATA
 #define ENODATA 9919
 #endif
-#if HAVE_SYS_RANDOM_H
-#include <sys/random.h>
-#endif
 
-#include <spa/utils/string.h>
 #include <spa/debug/types.h>
+#include <spa/utils/string.h>
 
 #include "pipewire/impl.h"
 #include "pipewire/private.h"
@@ -54,7 +49,8 @@ struct resource_data {
 static void * registry_bind(void *object, uint32_t id,
 		const char *type, uint32_t version, size_t user_data_size)
 {
-	struct pw_resource *resource = object;
+	struct resource_data *data = object;
+	struct pw_resource *resource = data->resource;
 	struct pw_impl_client *client = resource->client;
 	struct pw_context *context = resource->context;
 	struct pw_global *global;
@@ -98,7 +94,8 @@ error_exit_clean:
 
 static int registry_destroy(void *object, uint32_t id)
 {
-	struct pw_resource *resource = object;
+	struct resource_data *data = object;
+	struct pw_resource *resource = data->resource;
 	struct pw_impl_client *client = resource->client;
 	struct pw_context *context = resource->context;
 	struct pw_global *global;
@@ -113,7 +110,7 @@ static int registry_destroy(void *object, uint32_t id)
 	if (!PW_PERM_IS_R(permissions))
 		goto error_no_id;
 
-	if (id == PW_ID_CORE || !PW_PERM_IS_X(permissions))
+	if (id == PW_ID_CORE || !PW_PERM_CHECK(permissions, PW_PERM_X|PW_PERM_W))
 		goto error_not_allowed;
 
 	pw_log_debug("global %p: destroy global id %d", global, id);
@@ -186,7 +183,7 @@ static int core_hello(void *object, uint32_t version)
 
 	if (version >= 3) {
 		if ((res = pw_global_bind(client->global, client,
-				PW_PERM_ALL, PW_VERSION_CLIENT, 1)) < 0)
+				PW_PERM_ALL, PW_VERSION_CLIENT, PW_ID_CLIENT)) < 0)
 			return res;
 	}
 	return 0;
@@ -261,7 +258,7 @@ static struct pw_registry *core_get_registry(void *object, uint32_t version, siz
 	pw_resource_add_object_listener(registry_resource,
 				&data->object_listener,
 				&registry_methods,
-				resource);
+				data);
 
 	spa_list_append(&context->registry_resource_list, &registry_resource->link);
 
@@ -317,8 +314,10 @@ core_create_object(void *object,
 	if (!spa_streq(factory->info.type, type))
 		goto error_type;
 
-	if (factory->info.version < version)
-		goto error_version;
+	if (factory->info.version < version) {
+		pw_log_info("%p: version %d < %d", context,
+				factory->info.version, version);
+	}
 
 	if (props) {
 		properties = pw_properties_new_dict(props);
@@ -328,7 +327,8 @@ core_create_object(void *object,
 		properties = NULL;
 
 	/* error will be posted */
-	obj = pw_impl_factory_create_object(factory, resource, type, version, properties, new_id);
+	obj = pw_impl_factory_create_object(factory, resource, type,
+			version, properties, new_id);
 	if (obj == NULL)
 		goto error_create_failed;
 
@@ -339,7 +339,6 @@ error_no_factory:
 	pw_log_debug("%p: can't find factory '%s'", context, factory_name);
 	pw_resource_errorf_id(resource, new_id, res, "unknown factory name %s", factory_name);
 	goto error_exit;
-error_version:
 error_type:
 	res = -EPROTO;
 	pw_log_debug("%p: invalid resource type/version", context);
@@ -416,7 +415,7 @@ struct pw_impl_core *pw_context_create_core(struct pw_context *context,
 	this->info.host_name = pw_get_host_name();
 	this->info.version = pw_get_library_version();
 	do {
-		res = getrandom(&this->info.cookie,
+		res = pw_getrandom(&this->info.cookie,
 				sizeof(this->info.cookie), 0);
 	} while ((res == -1) && (errno == EINTR));
 	if (res == -1) {
@@ -584,6 +583,7 @@ int pw_impl_core_register(struct pw_impl_core *core,
 			 struct pw_properties *properties)
 {
 	static const char * const keys[] = {
+		PW_KEY_OBJECT_SERIAL,
 		PW_KEY_USER_NAME,
 		PW_KEY_HOST_NAME,
 		PW_KEY_CORE_NAME,
@@ -611,6 +611,8 @@ int pw_impl_core_register(struct pw_impl_core *core,
 
 	core->info.id = core->global->id;
 	pw_properties_setf(core->properties, PW_KEY_OBJECT_ID, "%d", core->info.id);
+	pw_properties_setf(core->properties, PW_KEY_OBJECT_SERIAL, "%"PRIu64,
+			pw_global_get_serial(core->global));
 	core->info.props = &core->properties->dict;
 
 	pw_global_update_keys(core->global, core->info.props, keys);
